__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from marketplacecli.utils import org_utils
from ussclicore.utils import printer
from ussclicore.utils import generics_utils
from marketplacecli.utils import marketplace_utils
from marketplace.objects.marketplace import *


class UserCmds(Cmd, CoreGlobal):
    """Manage users : list users, update users, update roles"""

    cmd_name = "user"

    def __init__(self):
        super(UserCmds, self).__init__()

    def arg_info(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " info", add_help=True,
                                   description="Displays informations of provided user")
        return do_parser

    def do_info(self, args):
        try:
            # call UForge API
            printer.out("Getting user [" + self.login + "] ...")
            user = self.api.Users(self.login).Get()

            if user is None:
                printer.out("user " + self.login + "does not exist", printer.ERROR)
            else:
                table = Texttable(200)
                table.set_cols_align(["c", "l", "c", "c", "c", "c", "c", "c"])
                table.header(
                    ["Login", "Email", "Lastname", "Firstname", "Created", "Active", "Promo Code", "Creation Code"])
                table.add_row([user.loginName, user.email, user.surname, user.firstName,
                               user.created.strftime("%Y-%m-%d %H:%M:%S"), "X", user.promoCode, user.creationCode])
                print table.draw() + "\n"
            return 0, user
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_info(self):
        do_parser = self.arg_info()
        do_parser.print_help()

    def arg_create(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " create", add_help=True, description="Creates a new user")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--account', dest='account', required=True, help="the login name of the user to create")
        mandatory.add_argument('--email', dest='email', required=True, help="the email of the user to create")
        mandatory.add_argument('--code', dest='code', required=True,
                               help="the creation code (subscription profile) to be used to create the user account")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--accountPassword', dest='accountPassword', required=False,
                              help="the new user account password")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization in which the user should be created")
        optional.add_argument('--disable', dest='disable', required=False,
                              help="flag to de-activate the account during the creation")
        return do_parser

    def do_create(self, args):
        try:
            # add arguments
            do_parser = self.arg_create()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            # call UForge API
            printer.out("Creating user account [" + do_args.account + "] ...")

            # create a user manually
            new_user = user()
            new_user.loginName = do_args.account
            new_user.password = do_args.accountPassword
            new_user.creationCode = do_args.code
            new_user.email = do_args.email
            new_user.password = do_args.accountPassword

            if do_args.org:
                org = do_args.org
            else:
                org = None
            if do_args.disable:
                new_user.active = False
            else:
                new_user.active = True

            # Send the create user request to the server
            new_user = self.api.Users(self.login).Create("true", "true", org, "false", "false", "false", new_user)

            if new_user is None:
                printer.out("No information about new user available", printer.ERROR)
            else:
                table = Texttable(200)
                table.set_cols_align(["c", "l", "c", "c", "c", "c", "c", "c"])
                table.header(
                    ["Login", "Email", "Lastname", "Firstname", "Created", "Active", "Promo Code", "Creation Code"])
                table.add_row([new_user.loginName, new_user.email, new_user.surname, new_user.firstName,
                               new_user.created.strftime("%Y-%m-%d %H:%M:%S"), "X", new_user.promoCode,
                               new_user.creationCode])
                print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_create(self):
        do_parser = self.arg_create()
        do_parser.print_help()

    def arg_list(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " list", add_help=True,
                                   description="List the users registered to the platform")
        return do_parser

    def do_list(self, args):
        try:

            # call UForge API
            printer.out("Getting users...")
            users = self.api.Users(self.login).Getall(None, None, None)

            if users is None:
                printer.out("No user")
            else:
                table = Texttable(200)
                table.set_cols_align(["c", "l", "c", "c", "c", "c", "c", "c"])
                table.header(
                    ["Login", "Email", "Lastname", "Firstname", "Created", "Active", "Promo Code", "Creation Code"])
                for u in users.users.user:
                    table.add_row([u.loginName, u.email, u.surname, u.firstName,
                                   u.created.strftime("%Y-%m-%d %H:%M:%S"), "X", u.promoCode, u.creationCode])
                print table.draw() + "\n"
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_list(self):
        do_parser = self.arg_list()
        do_parser.print_help()



__author__ = "UShareSoft"

from texttable import Texttable
from ussclicore.argumentParser import ArgumentParser, ArgumentParserError
from ussclicore.cmd import Cmd, CoreGlobal
from ussclicore.utils import printer
from marketplace.objects.marketplace import role
from marketplace.objects.marketplace import roles
from marketplacecli.utils import org_utils
from marketplacecli.utils import marketplace_utils
import pyxb


class SubscriptionProfileRoles(Cmd, CoreGlobal):
    """Manage subscription profile roles"""

    cmd_name = "role"

    def __init__(self):
        super(SubscriptionProfileRoles, self).__init__()

    def arg_add(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " add", add_help=True,
                                   description="Add roles to a subscription profile")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="the name of the subscription profile")
        mandatory.add_argument('--roles', dest='roles', nargs='+', required=True,
                               help="the roles to add to the subscription profile")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_add(self, args):
        try:
            # add arguments
            do_parser = self.arg_add()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Getting subscription profile with name [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(Search=do_args.name)

            subscription = subscriptions.subscriptionProfiles.subscriptionProfile[0]

            # Create the list of roles
            all_roles = roles()
            all_roles.roles = pyxb.BIND()

            # Copy the list of current administrators
            for r in subscription.roles.role:
                already_role = role()
                already_role.name = r.name
                all_roles.roles.append(already_role)

            # Add the new administrators given as input
            for nr in do_args.roles:
                new_role = role()
                new_role.name = nr
                all_roles.roles.append(new_role)

            # call UForge API
            self.api.Orgs(org.dbId).Subscriptions(subscription.dbId).Roles.Update(all_roles)
            printer.out("Some roles added for subscription profile [" + do_args.name + "]...")
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_add(self):
        do_parser = self.arg_add()
        do_parser.print_help()

    def arg_remove(self):
        do_parser = ArgumentParser(prog=self.cmd_name + " remove", add_help=True,
                                   description="Remove one or several roles from a subscription profile")
        mandatory = do_parser.add_argument_group("mandatory arguments")
        mandatory.add_argument('--name', dest='name', required=True, help="the name of the subscription profile")
        mandatory.add_argument('--roles', dest='roles', nargs='+', required=True,
                               help="the roles to add to the subscription profile")
        optional = do_parser.add_argument_group("optional arguments")
        optional.add_argument('--org', dest='org', required=False,
                              help="the organization name. If no organization is provided, then the default organization is used.")
        return do_parser

    def do_remove(self, args):
        try:
            # add arguments
            do_parser = self.arg_remove()
            try:
                do_args = do_parser.parse_args(args.split())
            except SystemExit as e:
                return

            printer.out("Getting subscription profile with name [" + do_args.name + "]...")
            org = org_utils.org_get(self.api, do_args.org)
            subscriptions = self.api.Orgs(org.dbId).Subscriptions().Getall(Search=do_args.name)

            subscription = subscriptions.subscriptionProfiles.subscriptionProfile[0]

            # Create the list of administrators
            all_roles = roles()
            all_roles.roles = pyxb.BIND()

            # Copy the list of current roles - Remove the roles selected in args
            for role in subscription.roles.role:
                if role.name not in do_args.roles:
                    already_role = role()
                    already_role.name = role.name
                    all_roles.roles.append(already_role)

            # call UForge API
            self.api.Orgs(org.dbId).Subscriptions(subscription.dbId).Roles.Update(all_roles)
            printer.out("Somes roles removed from subscription profile [" + do_args.name + "]...")
            return 0
        except ArgumentParserError as e:
            printer.out("ERROR: In Arguments: " + str(e), printer.ERROR)
            self.help_info()
        except Exception as e:
            return marketplace_utils.handle_uforge_exception(e)

    def help_remove(self):
        do_parser = self.arg_remove()
        do_parser.print_help()



"""
    UForge Marketplace CLI
"""

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

import getpass
import base64
import sys

import argparse
import httplib2
from marketplace.application import Api

from ussclicore.cmd import Cmd
from ussclicore.argumentParser import CoreArgumentParser, ArgumentParser, ArgumentParserError

from ussclicore.utils import printer
import commands
from utils import constants

__author__ = "UShareSoft"
__license__ = "Apache License 2.0"


class CmdBuilder(object):
    @staticmethod
    def generateCommands(class_):
        # Create subCmds if not exist
        if not hasattr(class_, 'subCmds'):
            class_.subCmds = {}

        # Add commands
        user = commands.usercmds.UserCmds()
        class_.subCmds[user.cmd_name] = user
        vendor = commands.vendorcmds.VendorCmds()
        class_.subCmds[vendor.cmd_name] = vendor
        subscription_profile = commands.subscriptionprofilecmds.SubscriptionProfileCmds()
        class_.subCmds[subscription_profile.cmd_name] = subscription_profile
        role = commands.rolecmds.RoleCmds()
        class_.subCmds[role.cmd_name] = role
        entitlement = commands.entitlementcmds.EntitlementCmds()
        class_.subCmds[entitlement.cmd_name] = entitlement


# Main cmd
class Mycli(Cmd):
    def __init__(self):
        super(Mycli, self).__init__()
        self.prompt = 'marketplacecli> '

        def do_exit(self, args):
            return True

        def do_quit(self, args):
            return True

        def arg_batch(self):
            do_parser = ArgumentParser("batch", add_help=True,
                                      description="Execute marketplacecli batch command from a file (for scripting)")
            mandatory = do_parser.add_argument_group("mandatory arguments")
            mandatory.add_argument('--file', dest='file', required=True, help="marketplacecli batch file commands")
            return do_parser

        def do_batch(self, args):
            try:
                do_parser = self.arg_batch()
                try:
                    do_args = do_parser.parse_args(args.split())
                except SystemExit as e:
                    return
                with open(do_args.file) as f:
                    for line in f:
                        try:
                            self.run_commands_at_invocation([line])
                        except:
                            printer.out("bad command '" + line + "'", printer.ERROR)
                        print "\n"

            except IOError as e:
                printer.out("File error: " + str(e), printer.ERROR)
                return
            except ArgumentParserError as e:
                printer.out("In Arguments: " + str(e), printer.ERROR)
                self.help_batch()

        def help_batch(self):
            do_parser = self.arg_batch()
            do_parser.print_help()

    def cmdloop(self, args):
        if len(args):
            code = self.run_commands_at_invocation([str.join(' ', args)])
            sys.exit(code)
        else:
            self._cmdloop()


def generate_base_doc(app, hep_app):
    myactions = []
    cmds = sorted(app.subCmds)
    for cmd in cmds:
        myactions.append(argparse._StoreAction(
            option_strings=[],
            dest=str(cmd),
            nargs=None,
            const=None,
            default=None,
            type=str,
            choices=None,
            required=False,
            help=str(app.subCmds[cmd].__doc__),
            metavar=None))

    return myactions


def set_globals_cmds(sub_cmds):
    for cmd in sub_cmds:
        if hasattr(sub_cmds[cmd], 'set_globals'):
            sub_cmds[cmd].set_globals(api, username, password)
            if hasattr(sub_cmds[cmd], 'subCmds'):
                set_globals_cmds(sub_cmds[cmd].subCmds)

# Generate marketplacecli base command + help base command
CmdBuilder.generateCommands(Mycli)
app = Mycli()
myactions = generate_base_doc(app, help="")


# Args parsing
mainParser = CoreArgumentParser(add_help=False)
CoreArgumentParser.actions = myactions
mainParser.add_argument('-a', '--url', dest='url', type=str, help='the server URL endpoint to use', required=False)
mainParser.add_argument('-u', '--user', dest='user', type=str, help='the user name used to authenticate to the server',
                        required=False)
mainParser.add_argument('-p', '--password', dest='password', type=str,
                        help='the password used to authenticate to the server', required=False)
mainParser.add_argument('-v', action='version', help='displays the current version of the marketplacecli tool',
                        version="%(prog)s version '" + constants.VERSION + "'")
mainParser.add_argument('-h', '--help', dest='help', action='store_true', help='show this help message and exit',
                        required=False)
mainParser.set_defaults(help=False)
mainParser.add_argument('cmds', nargs='*', help='Marketplace CLI cmds')
mainArgs, unknown = mainParser.parse_known_args()

if mainArgs.help and not mainArgs.cmds:
    mainParser.print_help()
    exit(0)

if mainArgs.user is not None and mainArgs.url is not None:
    if not mainArgs.password:
        mainArgs.password = getpass.getpass()
    username = mainArgs.user
    password = mainArgs.password
    url = mainArgs.url
    sslAutosigned = True
else:
    mainParser.print_help()
    exit(0)

# UForge API instanciation
client = httplib2.Http(disable_ssl_certificate_validation=sslAutosigned, timeout=constants.HTTP_TIMEOUT)
# activate http caching
# client = httplib2.Http(generics_utils.get_marketplacecli_dir()+os.sep+"cache")
headers = {'Authorization': 'Basic ' + base64.encodestring(username + ':' + password)}
api = Api(url, client=client, headers=headers)
set_globals_cmds(app.subCmds)

if mainArgs.help and len(mainArgs.cmds) >= 1:
    argList = mainArgs.cmds + unknown
    argList.insert(len(mainArgs.cmds) - 1, "help")
    app.cmdloop(argList)
elif mainArgs.help:
    app.cmdloop(mainArgs.cmds + unknown + ["-h"])
else:
    app.cmdloop(mainArgs.cmds + unknown)



import os
import sys

from setuptools import setup, find_packages

from marketplacecli.utils.constants import *

# Declare your packages' dependencies here, for eg:
requires = ['uforge-marketplace_python_sdk>=3.0, <3.1',
            'httplib2==0.9',
            'cmd2==0.6.7',
            'texttable==0.8.1',
            'progressbar==2.3',
            'argparse',
            'pyparsing==2.0.2',
            'hurry.filesize==0.9',
            'termcolor==1.1.0',
            'xmlrunner==1.7.7',
            'ussclicore==1.0.1']

if os.name != "nt":
    if "linux" not in sys.platform:
        # mac os
        requires.append('readline')
else:  # On Windows
    requires.append('pyreadline==2.0')

setup(

    install_requires=requires,

    # Fill in these to make your Egg ready for upload to
    # PyPI
    name='marketplacecli',
    version=VERSION,
    description='',
    long_description='',
    packages=find_packages(),
    author='UShareSoft',
    author_email='contac@usharesoft.com',
    license="Apache License 2.0",
    url='',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ),

    # long_description= 'Long description of the package',
    scripts=['bin/marketplacecli', 'bin/marketplacecli.bat'],

)
